/*---------------------------------------------------------------------------*
 |          Copyright (c) 1997-2010 PDFlib GmbH. All rights reserved.        |
 +---------------------------------------------------------------------------+
 |    This software may not be copied or distributed except as expressly     |
 |    authorized by PDFlib GmbH's general license agreement or a custom      |
 |    license agreement signed by PDFlib GmbH.                               |
 |    For more information about licensing please refer to www.pdflib.com.   |
 *---------------------------------------------------------------------------*/

/*
 * $Id: pcoslibdl.c,v 1.2 2010/09/07 13:19:32 stm Exp $
 *
 * C wrapper code for dynamically loading the pCOS DLL at runtime.
 *
 * This module is not supported on all platforms.
 */

#include <stdlib.h>
#include <stdio.h>

#include "pcoslibdl.h"

/* enable this to avoid error messages */
/*
#define PDF_SILENT
*/

#ifdef WIN32

#define PCOS_DLLNAME                     "libpcos.dll"

/* ---------------------------------- MVS ----------------------------- */

#elif defined(__MVS__)

#define PCOS_DLLNAME                     "PCOS"

/* ---------------------------------- Linux  ----------------------------- */

#elif defined(__linux__) || defined(linux)

#define PCOS_DLLNAME                     "libpcos.so"

/* ---------------------------------- Mac OS X  ----------------------------- */

#elif defined(__APPLE__)

#define PCOS_DLLNAME                     "libpcos.dylib"

/* ---------------------------------- AS/400  ----------------------------- */

#elif defined __ILEC400__

#define PCOS_DLLNAME                     "PCOS"

/* ---------------------------------- unknown  ----------------------------- */

#else

#error No DLL loading code for this platform available!

#endif

#include "dl_dl.h"

static void
pCOS_dlerror(const char *msg)
{
#ifndef PDF_SILENT
    fprintf(stderr, msg);
#endif
}

/* Load the pCOS DLL and fetch the API structure */
PDFLIB_API const pCOS_api * PDFLIB_CALL
pCOS_new_dl(pCOS **p_pcos_obj)
{
    const pCOS_api *pcos_api, *(PDFLIB_CALL *get_api)(void);
    char buf[256];

    /* load the pCOS DLL... */
    void *handle = pdfdl_dlopen(PCOS_DLLNAME);

    if (!handle)
    {
	pCOS_dlerror("Error: couldn't load pCOS DLL\n");
	return NULL;
    }

    /* ...and fetch function pointers */
    get_api = (const pCOS_api *(PDFLIB_CALL *)(void))
		pdfdl_dlsym(handle, "pCOS_get_api");

    if (get_api == NULL)
    {
	pCOS_dlerror(
	    "Error: couldn't find function pCOS_get_api in pCOS DLL\n");
	pdfdl_dlclose(handle);
	return NULL;
    }

    /* Fetch the API structure. */
    pcos_api = (*get_api)();

    /*
     * Check the version number of the loaded DLL against that of
     * the included header file to avoid version mismatch.
     */
    if (pcos_api->sizeof_pCOS_api != sizeof(pCOS_api) ||
            pcos_api->major != PCOS_MAJORVERSION ||
            pcos_api->minor != PCOS_MINORVERSION) {
	sprintf(buf,
	"Error: loaded wrong version of pCOS DLL\n"
	"Expected version %d.%d (API size %lu), loaded %d.%d (API size %lu)\n",
	PCOS_MAJORVERSION, PCOS_MINORVERSION, (unsigned long) sizeof(pCOS_api),
	pcos_api->major, pcos_api->minor,
	(unsigned long) pcos_api->sizeof_pCOS_api);
	pCOS_dlerror(buf);
	pdfdl_dlclose(handle);
	return NULL;
    }

    /*
     * Create a new pCOS object; use pCOS_new2() so that we can store
     * the DLL handle within the pCOS object and later retrieve it.
     */
    if ((*p_pcos_obj = pcos_api->pCOS_new2(NULL, handle)) == (pCOS *) NULL)
    {
        pCOS_dlerror("Couldn't create pCOS object (out of memory)!\n");
        pdfdl_dlclose(handle);
        return NULL;
    }

    return pcos_api;
}

/* delete the PCOS object and unload the previously loaded pCOS DLL */
PDFLIB_API void PDFLIB_CALL
pCOS_delete_dl(const pCOS_api *pcos_api, pCOS *pcos)
{
    if (pcos_api && pcos)
    {
        /* fetch the DLL handle (previously stored in the pCOS object) */
        void *handle = pcos_api->pCOS_get_opaque(pcos);

        pcos_api->pCOS_delete(pcos);

        if (handle)
            pdfdl_dlclose(handle);
    }
}
